import unittest
from datetime import date, datetime
from unittest.mock import Mock

import numpy as np
import pandas as pd
from common.utils.common_model import Interval
from common.utils.flow.core import FlowService
from common.utils.flow.data import FlowGroupByQuery
from common.utils.lookup.core import LookupService


class TestCore(unittest.TestCase):
    def setUp(self):
        self._mock_flow_service = Mock(FlowService)
        self.lookup_service = LookupService(self._mock_flow_service)

    def test_computes_to_ip_lookup(self):
        self._mock_flow_service.group_by = mock_group_by
        result = self.lookup_service.computes_to_ip_lookup("default", Interval(datetime.combine(
            date.today(), datetime.min.time()), datetime.now()))

        self.assertEqual("192.168.1.1", result["vm1"])
        self.assertEqual("192.168.1.2", result["vm2"])
        self.assertEqual("192.168.0.3", result["vm3"])
        self.assertEqual("192.168.0.4", result["vm4"])
        self.assertTrue("vm5" not in result)
        self.assertEqual("192.168.1.3", result["vm6"])
        self.assertEqual("192.168.1.4", result["vm7"])
        self.assertTrue("vm8" not in result)
        self.assertEqual(6, len(result))


def mock_group_by(query: FlowGroupByQuery) -> pd.DataFrame:
    src_result_data = [
        [
            ["vm1", "192.168.0.1", datetime(year=2023, month=1, day=1)],
            ["vm2", "192.168.0.2", datetime(year=2023, month=1, day=1)],
            ["vm3", "192.168.0.3", datetime(year=2023, month=1, day=1)],
            ["vm4", "192.168.0.4", datetime(year=2023, month=1, day=1)],
            ["vm5", np.nan, datetime(year=2023, month=1, day=1)]
        ]
    ]
    src_results = [pd.DataFrame(
        data, columns=["srcVmId", "ipAddress", "timestamp"]) for data in src_result_data]

    dst_result_data = [
        [
            ["vm1", "192.168.1.1", datetime(year=2023, month=1, day=1)],
            ["vm2", "192.168.1.2", datetime(year=2023, month=1, day=1)],
            ["vm6", "192.168.1.3", datetime(year=2023, month=1, day=1)],
            ["vm7", "192.168.1.4", datetime(year=2023, month=1, day=1)],
            ["vm8", np.nan, datetime(year=2023, month=1, day=1)]
        ]
    ]
    dst_results = [pd.DataFrame(
        data, columns=["dstVmId", "ipAddress", "timestamp"]) for data in dst_result_data]

    if query.dimensions == ["srcVmId"]:
        return src_results[0]

    if query.dimensions == ["dstVmId"]:
        return dst_results[0]

    raise Exception("mocked group by does not support the given query")
