import unittest

from common.utils.vizfilter.core import (
    CompositeVizFilter,
    Condition,
    Criteria,
    VizFilter,
    VizFilterOption,
)


class TestCore(unittest.TestCase):
    def test_VizFilter(self):
        viz_filter = VizFilter(
            VizFilterOption.VM_ID,
            ["vm1", "vm2"],
            Criteria.EQ,
            Condition.OR
        )

        expected_dict = {
            "key": "VM_ID",
            "values": ["vm1", "vm2"],
            "criteria": "EQ",
            "condition": "OR"
        }

        self.assertEqual(expected_dict, viz_filter.to_dict())

    def test_CompositeVizFilter(self):
        viz_filter_1 = VizFilter(
            VizFilterOption.VM_ID,
            ["vm1", "vm2"],
            Criteria.EQ,
            Condition.OR
        )
        viz_filter_2 = VizFilter(
            VizFilterOption.FLOW_DSTPORT,
            ["80", "443"],
            Criteria.EQ,
            Condition.OR
        )

        composite_viz_filter = CompositeVizFilter(
            viz_filters=[viz_filter_1, viz_filter_2],
            condition=Condition.AND
        )

        expected_dict = {
            "condition": "AND",
            "vizFilters": [
                {
                    "key": "VM_ID",
                    "values": ["vm1", "vm2"],
                    "criteria": "EQ",
                    "condition": "OR"
                },
                {
                    "key": "FLOW_DSTPORT",
                    "values": ["80", "443"],
                    "criteria": "EQ",
                    "condition": "OR"
                }
            ]
        }

        self.assertEqual(expected_dict, composite_viz_filter.to_dict())
