from dataclasses import dataclass
from datetime import datetime, timedelta
from enum import Enum, auto


class DetectorType(Enum):
    NETWORK_TRAFFIC_DROP = 0
    HORIZONTAL_PORT_SCAN = auto()
    VERTICAL_PORT_SCAN = auto()
    UNCOMMONLY_USED_PORT = auto()
    LLMNR_NBTNS = auto()
    REMOTE_SERVICES = auto()
    FLOW_BEACONING = auto()
    DEST_IP_PROFILER = auto()
    SERVER_PORT_PROFILER = auto()
    PORT_PROFILER = auto()
    UPDOWNLOADER_TIMESERIES = auto()
    DNS_TUNNELING = auto()
    DGA = auto()
    DATA_UPDOWNLOADER = auto()


@dataclass
class Interval:
    start_time: datetime
    end_time: datetime

    def delta(self) -> timedelta:
        return self.end_time - self.start_time

    def isoformat(self) -> str:
        return f"{self.start_time.isoformat()}/{self.end_time.isoformat()}"
