from confluent_kafka import Producer

from common.utils import logging
from common.utils.event import data
from common.utils.event.config import Config


class EventError(Exception):
    pass


class EventService:
    logger = logging.get_common_logger(__name__)

    def __init__(self):
        self._data = data.Data(Producer({
            "bootstrap.servers": Config.KAFKA_BOOTSTRAP_SERVER,
            "security.protocol": Config.KAFKA_SECURITY_PROTOCOL,
            "ssl.ca.location": Config.KAFKA_SSL_CA_LOCATION,
            "ssl.certificate.location": Config.KAFKA_SSL_CERTIFICATE_LOCATION,
            "ssl.key.location": Config.KAFKA_SSL_KEY_LOCATION,
            "ssl.key.password": Config.KAFKA_SSL_KEY_PASSWORD
        }))

    def publish_event(self, event: data.EventInfo) -> None:
        self.publish_events([event])

    def publish_events(self, events: list[data.EventInfo]) -> None:
        try:
            EventService.logger.info(f"publishing {len(events)} events")
            self._data.produce_events(events)
        except Exception as e:
            raise EventError("publishing events failed") from e
