from dataclasses import dataclass

import psycopg
from psycopg import sql
from psycopg.rows import class_row

from common.utils import logging
from common.utils.common_model import Interval
from common.utils.lookup.config import Config


@dataclass
class GroupComputeRelationshipConfigEntity:
    computeid: str
    groupid: str
    siteid: str
    version: int
    create_time: int
    delete_time: int


@dataclass
class GroupComputeRelationshipConfigInfo:
    compute_id: str
    group_id: str
    site_id: str
    version: int
    create_time: int
    delete_time: int


class Data:
    logger = logging.get_common_logger(__name__)

    def find_associated_computes_by_interval_and_site(
        self, group_id: str, interval: Interval, site_id: str
    ) -> list[GroupComputeRelationshipConfigInfo]:
        connection_string = "dbname={dbname} user={user} password={password} host={host} port={port}".format(
            dbname=Config.POSTGRES_DB_NAME,
            user=Config.POSTGRES_USER,
            password=Config.POSTGRES_PASSWORD,
            host=Config.POSTGRES_HOST,
            port=Config.POSTGRES_PORT,
        )
        with psycopg.connect(connection_string) as conn:
            with conn.cursor(row_factory=class_row(GroupComputeRelationshipConfigEntity)) as cur:
                query_string = (
                    "select * from {table} where {site_id} = %(site_id)s and {group_id} = %(group_id)s and "
                    "( ({create_time} between %(start_time)s and %(end_time)s or {delete_time} between %(start_time)s and %(end_time)s) or "
                    "({delete_time} = 0 and {create_time} <= %(start_time)s) )"
                )
                query = sql.SQL(query_string).format(
                    table=sql.Identifier(Config.GROUP_COMPUTE_RELATIONSHIP_CONFIG_TABLE),
                    site_id=sql.Identifier("siteid"),
                    group_id=sql.Identifier("groupid"),
                    create_time=sql.Identifier("create_time"),
                    delete_time=sql.Identifier("delete_time"),
                )

                values = {
                    "site_id": site_id,
                    "group_id": group_id,
                    "start_time": int(interval.start_time.timestamp() * 1000),
                    "end_time": int(interval.end_time.timestamp() * 1000),
                }
                cur.execute(query, values)

                group_compute_relationship_config_infos = [self._entity_to_info(record) for record in cur]
                return group_compute_relationship_config_infos

    def _entity_to_info(self, entity: GroupComputeRelationshipConfigEntity) -> GroupComputeRelationshipConfigInfo:
        return GroupComputeRelationshipConfigInfo(
            entity.computeid, entity.groupid, entity.siteid, entity.version, entity.create_time, entity.delete_time
        )
