from common.utils import logging
from common.utils.userconfig import data
from common.utils.userconfig.data import DetectorRunStatus


class UserConfigError(Exception):
    pass


class DetectorRunStatusError(Exception):
    pass


class UserConfigService:
    logger = logging.get_common_logger(__name__)

    def __init__(self):
        self._data = data.Data()

    def get_user_config(self, detector_name: str, org_id: str) -> list[data.UserConfigInfo]:
        if not detector_name or not org_id:
            raise UserConfigError(f"name and org_id must be provided, got name: {detector_name}, org_id: {org_id}")
        try:
            UserConfigService.logger.info(f"retrieving user configs for {detector_name}")
            return self._data.get_user_config({"name": detector_name, "org_id": org_id})
        except Exception as e:
            raise UserConfigError("an error occurred fetching the user config") from e

    def update_detector_status(self, status: DetectorRunStatus, detector_name: str, org_id: str, site_id: str) -> None:
        if not detector_name or not org_id or not site_id:
            raise UserConfigError(
                f"name, org_id and site_id must be provided, got name: {detector_name}, org_id: {org_id}, site_id: {site_id}"
            )
        try:
            UserConfigService.logger.info(f"updating detector status {detector_name} {org_id} {site_id}")
            self._data.update_detector_status(status, {"name": detector_name, "org_id": org_id, "site_id": site_id})
        except Exception as e:
            raise UserConfigError("an error occurred updating the detector status") from e

    def update_detector_status_all_sites(self, status: DetectorRunStatus, detector_name: str, org_id: str) -> None:
        if not detector_name or not org_id:
            raise UserConfigError(
                f"name, org_id and site_id must be provided, got name: {detector_name}, org_id: {org_id}"
            )
        try:
            UserConfigService.logger.info(f"updating detector status label for {detector_name} {org_id}")
            self._data.update_detector_status(status, {"name": detector_name, "org_id": org_id})
        except Exception as e:
            raise UserConfigError("an error occurred updating the detector status label") from e
