import json
from dataclasses import dataclass
from enum import Enum, StrEnum, auto
from typing import Any


class Criteria(Enum):
    EQ = 0


class Condition(Enum):
    OR = 0
    AND = auto()


class VizFilterOption(StrEnum):
    FLOW_PROTOCOL = "flow.protocol"
    FLOW_DSTPORT = "flow.dstPort"
    FLOW_SRCIP = "flow.srcIp"
    FLOW_DSTIP = "flow.dstIp"
    FLOW_TYPE = "flow.type"
    FLOW_APPID = "flow.applicationId"
    FLOW_STATUS = "flow.status"
    FLOW_SERVICE_NAME = "flow.service.name"
    VM_IP = "vms.ip_addresses"
    VM_NAME = "vms.name"
    VM_OS = "vms.os_info"
    VM_ID = "vms.id"
    VM_TAGS = "vms.tags"


@dataclass
class VizFilter:
    key: VizFilterOption
    values: list[str]
    criteria: Criteria
    condition: Condition

    ALLOWED = "ALLOWED"
    BLOCKED = "BLOCKED"

    def to_dict(self) -> dict[str, Any]:
        return {
            "key": self.key.name,
            "values": self.values,
            "criteria": self.criteria.name,
            "condition": self.condition.name
        }

    def to_json_str(self) -> str:
        return json.dumps(self.to_dict())

    def __repr__(self) -> str:
        return "VizFilter{{key={}, values={}, criteria={}, condition={}}}".format(
            self.key, self.values, self.criteria, self.condition)

    def __str__(self) -> str:
        return self.__repr__()


@dataclass
class CompositeVizFilter:
    viz_filters: list[VizFilter]
    condition: Condition

    def to_dict(self) -> dict[str, Any]:
        return {
            "condition": self.condition.name,
            "vizFilters": [filter.to_dict() for filter in self.viz_filters]
        }

    def to_json_str(self) -> str:
        return json.dumps(self.to_dict())

    def __repr__(self) -> str:
        return "CompositeVizFilter{{vizFilters={}, condition={}}}".format(
            self.viz_filters, self.condition.name)

    def __str__(self) -> str:
        return self.__repr__()
